import os
import unittest
from envs_vars import *
import requests
from selenium import webdriver
from splinter import Browser

# For fixtures (environ variables) see admin module

# and selenium needs a browser driver somewhere in PATH
os.environ['PATH'] = os.environ['PATH'] + ":/opt/apps/selenium-driver"


def find_button_by_text(browser, text):
    return browser.find_by_xpath('//*[contains(@class, "dijitButtonText") and text() = \'%s\']' % text)


def find_exception_button_by_text(browser, text):
    return browser.find_by_xpath('//*[contains(@class, "dijitButtonNode")]contains(@class, "dijitReset")]')


def auth():
    return get_auth_token(os.environ['TESTPRIVACYIDEA_URL'], os.environ['TESTPRIVACYIDEA_USER'], os.environ['TESTPRIVACYIDEA_PASS'])


def get_auth_token(url, username, password):
    r = requests.post(url + "/auth", json={'username': username, 'password': password})
    if r.status_code != 200:
        raise ValueError("wrong credentials")

    auth_token = r.json()["result"]["value"]["token"]
    if auth_token is None:
        raise ValueError("couldn't extract auth token")

    return auth_token


def delete_all_tokens_for_user(url, auth_token, user):
    r = requests.get(url + "/token", headers={'Authorization': auth_token}, json={'user': user})
    result = r.json()['result']

    if r.status_code == 200:
        for tfa_token in result['value']['tokens']:
            r = requests.delete(url + "/token/%s" % tfa_token['serial'], headers={'Authorization': auth_token})
            if r.status_code != 200:
                raise ValueError("could not delete token")


class TestGui(unittest.TestCase):
    def setUp(self):
        super(unittest.TestCase, self)
        self.auth_token = auth()
        delete_all_tokens_for_user(os.environ['TESTPRIVACYIDEA_URL'], self.auth_token, os.environ['TESTUSER1'])

    def login(self, username, password, otp=None):

        # profile = webdriver.FirefoxProfile()
        # profile.accept_untrusted_certs = True
        # driver = webdriver.Firefox(firefox_profile=profile)
        # browser = Browser('firefox', firefox_driver=driver)

        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument('ignore-certificate-errors')
        browser = Browser('chrome', chrome_options=chrome_options)
        browser.driver.implicitly_wait(10)

        if otp:
            browser.visit('%s/univention/saml' % os.environ['TESTURL'])
        else:
            browser.visit('%s/univention/login' % os.environ['TESTURL'])

        browser.find_by_id('umcLoginUsername').fill(username)
        browser.find_by_id('umcLoginPassword').fill(password)
        browser.find_by_name('submit').click()

        self.assertTrue(browser.is_element_present_by_css('.umcGalleryItem'), "Login failed")
        return browser

    def testTokenActions(self):
        # test initial generation
        browser = self.login(os.environ['TESTUSER1'], os.environ['TESTUSER1PASS'])
        browser.visit("%s/univention/management/#module=twofactor-user::0:" % os.environ['TESTURL'])
        self.assertTrue(browser.is_element_present_by_css('.dijitDialogTitle'), "Confirmation")
        find_button_by_text(browser, 'Continue').click()
        self.assertTrue(browser.is_element_present_by_css('[data-qa=qrcode-image]'), "No QR-Code Image")

        # try second time and fail
        find_button_by_text(browser, 'Close').click()
        browser.visit("%s/univention/management/#module=twofactor-user::0:" % os.environ['TESTURL'])
        self.assertTrue(browser.is_element_present_by_css('.dijitDialog'), "Confirmation")
        self.assertTrue(browser.is_text_present("A hardware or software token has already been created for your user account."))
        find_button_by_text(browser, 'OK').click()
        pass

if __name__=="__main__":
    unittest.main()