"""

    This tests the usc jsonrpc implementation, doing the same stuff dojo will do eventually.
    The reasoning is faster development / divide an conquer: First get the backend right, then worry about the frontend

    For fixtures see admin module

"""
import unittest
from os import environ
from envs_vars import *
import requests

baseurl = "%s/univention/" % environ['TESTURL']
#baseurl = "%s/" % environ['TESTURL']
twofactor_baseurl = baseurl + "command/twofactor-user/"
# proxies = {"http": "http://localhost:8079", "https": "https://localhost:8079"}
proxies = {}


def search(uid, userlist):
    return [element for element in userlist if element['uid'] == uid]


class TestRolloverToken(unittest.TestCase):
    cookies = None
    headers = None

    privacyidea_url = environ['TESTPRIVACYIDEA_URL']

    privacyidea_auth_token = None

    @classmethod
    def setUpClass(self):
        r = requests.post(self.privacyidea_url + "/auth", json={'username': environ['TESTPRIVACYIDEA_USER'], 'password': environ['TESTPRIVACYIDEA_PASS']}, verify=False)
        if r.status_code != 200:
            self.fail("wrong credentials")

        self.privacyidea_auth_token = r.json()["result"]["value"]["token"]
        if self.privacyidea_auth_token is None:
            self.fail("couldn't extract auth token")

    def delete_tokens_for_user(self, username):
        # search all tokens for teacher
        r = requests.get(self.privacyidea_url + "/token", headers={'Authorization': self.privacyidea_auth_token}, json={'user': username}, verify=False)
        result = r.json()['result']
        if r.status_code == 200 and result['value']['count'] > 0:
            for tfa_token in result['value']['tokens']:
                r = requests.delete(self.privacyidea_url + "/token/%s" % tfa_token['serial'], headers={'Authorization': self.privacyidea_auth_token})
                if r.status_code != 200:
                    self.fail("could not delete old user tokens")

    @classmethod
    def login(self, user, password):
        res = requests.post(baseurl + "auth", json={"options": {"username": user, "password": password}},
                            cookies={"UMCLang": "en-US"}, proxies=proxies, verify=False)
        assert 200 == res.status_code
        assert res.cookies is not None
        self.cookies = res.cookies
        # pin language since some error messages are translated on the server
        requests.utils.add_dict_to_cookiejar(self.cookies, {"UMCLang": "en-US"})
        self.headers = {'X-Xsrf-Protection': res.cookies.get("UMCSessionId")}

    def logout(self):
        res = requests.get(baseurl + "/logout", cookies=self.cookies, headers=self.headers, proxies=proxies)
        self.assertEqual(200, res.status_code)

    def assert_token_actions(self, username, password):
        self.login(username, password)
        # delete token for user
        self.delete_tokens_for_user(username)
        url = twofactor_baseurl + "check_token_status"
        res = requests.post(url, cookies=self.cookies, headers=self.headers)
        self.assertEqual(200, res.status_code)
        self.assertEqual("user_has_no_token(s)", res.json()['result']['state'])

        # call function and get a new token
        url = twofactor_baseurl + "generate_inital_qr_token"
        res = requests.post(url, cookies=self.cookies, headers=self.headers)
        self.assertEqual(200, res.status_code)
        self.assertEqual("initial_token", res.json()['result']['type'])
        self.assertIsNotNone(res.json()['result']['image'])

        # call status again and ensure user has a token
        url = twofactor_baseurl + "check_token_status"
        res = requests.post(url, cookies=self.cookies, headers=self.headers)
        self.assertEqual(200, res.status_code)
        self.assertEqual("user_has_token(s)", res.json()['result']['state'])

        url = twofactor_baseurl + "generate_inital_qr_token"
        # call function again and get an error because there already is a token
        res = requests.post(url, cookies=self.cookies, headers=self.headers)
        self.assertEqual(400, res.status_code)

    def test_testuser(self):
        self.assert_token_actions(environ['TESTUSER1'], environ['TESTUSER1PASS'])

    def test_admin(self):
        self.assert_token_actions("Administrator", environ['TESTADMINPASS'])

    def test_domainadmin(self):
        self.assert_token_actions(environ['TESTDOMAINADMIN'], environ['TESTDOMAINADMINPASS'])




if __name__ == '__main__':
    unittest.main()
